package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;

import java.util.List;


/**
 * 布尔型解析器
 * <p>
 * 不支持区间
 * <p>
 * 假设信息点编码的值为Info01, 入参格式:<br/>
 * 单数 非区间: { "Info01": 0 }<br/>
 * 复数 非区间: { "Info01": [ 0, 1, 0] }<br/>
 */
public class BooleanParser extends AbstractParser<Integer> {

	@Override
	public void checkDef(ParserContext context) {
		if (context.getDef().getIsRegion()) {
			context.addError("Boolean类型不支持区间");
		}
	}

	@Override
	public void checkValue(ParserContext<Integer> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();
		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		if (isRegion) {
			// 区间
			context.addError("不支持[Boolean 区间]");
			return;
		}
		if (isMultiple) {
			String error = "值错误, 需要[ 0 / 1 ]数组, 传入[" + sourceStr + "]";
			// 复数
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}
			List<Integer> value = ParserUtils.parseToList(sourceStr, Integer.class);
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			for (Integer v : value) {
				if (v != 0 && v != 1) {
					context.addError(error);
					return;
				}
			}
			context.setArrayValue(value);
		} else {
			String error = "值错误, 需要[ 0 / 1 ], 传入[" + sourceStr + "]";
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError(error);
				return;
			}
			Integer value = ParserUtils.parse(sourceStr, Integer.class);
			if (value == null) {
				context.addError(error);
				return;
			}
			if (value != 0 && value != 1) {
				context.addError(error);
				return;
			}
			context.setValue(value);
		}
	}
}
